package net.wanji.web.service.impl;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.ControlModeEnum;
import net.wanji.common.framework.Constants;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.vo.*;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.mapper.ControlHistMapper;
import net.wanji.web.service.FeignProxyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2023/02/09 10:44
 */
@Service
@Slf4j
@SuppressWarnings("all")
public class FeignProxyServiceImpl implements FeignProxyService {

    @Autowired
    private UtcFeignClients utcFeignClients;
    @Autowired
    private ControlHistMapper controlHistMapper;

    @Override
    public JsonViewObject schemeSend(SchemeSendVO schemeSendVO) {
        JsonViewObject jsonViewObject = utcFeignClients.schemeSend(schemeSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-基础方案下发UTC服务调用异常");
        }
        return jsonViewObject.success("信号机方案下发成功");
    }

    @Override
    public JsonViewObject planSend(PlanSendVO planSendVO) {
        JsonViewObject jsonViewObject = utcFeignClients.planSend(planSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("计划下发-计划信息、时段信息UTC服务调用异常");
        }
        return jsonViewObject.success("计划下发-计划信息、时段信息下发成功");
    }

    @Override
    public JsonViewObject scheduleSend(ScheduleSendVO scheduleSendVO) {
        JsonViewObject jsonViewObject = utcFeignClients.scheduleSend(scheduleSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("时间表下发-时间表信息UTC服务调用异常");
        }
        return jsonViewObject.success("时间表下发-时间表信息下发成功");
    }

    @Override
    public JsonViewObject phaseTimingSend(PhaseTimingSendVO phaseTimingSendVO) {
        JsonViewObject jsonViewObject = utcFeignClients.phaseTimingSend(phaseTimingSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("相位配时下发-（下发相位参数）UTC服务调用异常");
        }
        return jsonViewObject.success("相位配时下发-相位参数下发成功");
    }

    @Override
    public JsonViewObject allRedControl(ControlCommandVO commandVO) {
        String resultMessage = "";
        Integer controlType = commandVO.getControlType();
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (1 == controlType) {
            jsonViewObject = utcFeignClients.allRedControl(commandVO);
            resultMessage = ControlModeEnum.FULL_RED.getName();
        }
        if (2 == controlType) {
            jsonViewObject = utcFeignClients.yellowLightControl(commandVO);
            resultMessage = ControlModeEnum.YELLOW_FLASH.getName();
        }
        if (3 == controlType) {
            jsonViewObject = utcFeignClients.closeLightControl(commandVO);
            resultMessage = ControlModeEnum.OFF_LIGHT.getName();
        }
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("全红黄闪灯态控制UTC服务调用异常");
        }
        Integer isnertControl = null;
        if (Objects.equals(0, commandVO.getCommand())) {
            isnertControl = 1;
        } else {
            isnertControl = controlType + 1;
        }
        insertControlHist(commandVO.getCrossCode(), isnertControl, Constants.ProcessService.ADMIN_ID);
        return jsonViewObject.success(resultMessage.concat("控制成功"));
    }

    @Override
    public JsonViewObject stepControl(StepControlVO stepControlVO) {
        String resultMessage = "步进控制成功";
        Integer control = 7;
        if (stepControlVO.getCommand() == 0) {
            control = 1;
            resultMessage = "步进控制恢复成功";
        }
        JsonViewObject jsonViewObject = utcFeignClients.stepControl(stepControlVO.getCode(), stepControlVO.getCommand(), stepControlVO.getStepNum());
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("步进控制-步进控制/恢复UTC服务调用异常");
        }
        insertControlHist(stepControlVO.getCode(), control, Constants.ProcessService.ADMIN_ID);
        return jsonViewObject.success(resultMessage);
    }

    @Override
    public JsonViewObject recoverSchedule(String code) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        jsonViewObject = utcFeignClients.recoverSchedule(code);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("恢复时间表-恢复路口时间表执行UTC服务调用异常");
        }
        insertControlHist(code, 1, Constants.ProcessService.ADMIN_ID);
        return jsonViewObject.success("恢复时间表-恢复路口时间表成功");
    }

    @Override
    public JsonViewObject lockControl(LockControlVO lockControlVO) {
        String resultMessage = "锁定控制-相位锁定控制成功";
        Integer control = 4;
        if (lockControlVO.getCommand() == 0) {
            resultMessage = "锁定控制-相位锁定恢复成功";
            control = 1;
        }
        JsonViewObject jsonViewObject = utcFeignClients.lockControl(lockControlVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("锁定控制-相位锁定/解锁UTC服务调用异常");
        }
        insertControlHist(lockControlVO.getCrossCode(), control, Constants.ProcessService.ADMIN_ID);
        return jsonViewObject.success(resultMessage);
    }

    @Override
    public JsonViewObject phaseDiffSend(HttpServletRequest httpServletRequest) {
        JsonViewObject jsonViewObject = utcFeignClients.phaseDiffSend();
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("相位差下发UTC服务调用异常");
        }
        return jsonViewObject.success("相位差下发成功");
    }

    @Override
    public JsonViewObject delBaseConfig(DelBaseConfigVO delBaseConfigVO) {
        JsonViewObject jsonViewObject = utcFeignClients.delBaseConfig(delBaseConfigVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("删除信号机参数UTC服务调用异常");
        }
        return jsonViewObject.success("删除信号机参数成功");    }

    /**
     * 特殊控制插入本地控制历史表
     *
     * @param crossCode
     * @param type
     * @param userName
     */
    private void insertControlHist(String crossCode, Integer type, String userName) {
        controlHistMapper.insertOne(crossCode, type, userName);
    }
}
